/***************************************************************************
                          gifblocks.cpp  -  description
                             -------------------
    begin                : Thu Jun 15 2000
    copyright            : (C) 2000 by Helmut Fahrion
    email                : hefa@bitctrl.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "gifblocks.h"

// ********* Liste *******************
liste::liste (liste * l)
{
  if (l)
    l->next = this;		// Vorgnger zeigt auf mich
  next = NULL;			// gibs noch nicht
  prev = l;			// mein Vorgnger ist das
}

liste::~liste (void)
{
  delete (next);
}

// ********* GifBlock *******************
gifblock::gifblock (gifblock * l):liste (l)
{
}

gifblock::~gifblock ()
{
}

// ********* Colormap *******************
colormap::colormap (ColorMapObject * cm)
{
  //   cm->BitsPerPixel  ist uninterresant ist meist 8
  len = cm->ColorCount;
  bpl = cm->BitsPerPixel;
  c = NULL;

  if (len)
    {
      c = new color[len];

      if (!c)
	throw (-1);

      for (int i = 0; i < len; i++)
	{
	  c[i].r = (unsigned char) cm->Colors[i].Red;
	  c[i].g = (unsigned char) cm->Colors[i].Green;
	  c[i].b = (unsigned char) cm->Colors[i].Blue;
	}
    }
  else
    throw (-1);

}

colormap::~colormap ()
{
  if (c)
    delete[]c;
}

// ********* ScreenBlock *******************
void
screenblock::draw (void *widget, void *win)
{
  // zeichnet nix
}

unsigned int
screenblock::getnr ()
{
  return 0;
}

screenblock::screenblock (gifblock * l, char *dateiname, int screenh,
			  int screenw, int screenc, int screenb,
ColorMapObject * screencm):
gifblock (l)
{
  strcpy (dname, dateiname);

  sw = screenw;
  sh = screenh;
  scr = screenc;
  sbc = screenb;
  cmap = NULL;

  if (screencm)
    {
      cmap = new colormap (screencm);	// Anzahl der Farben
      if (!cmap)
	throw (-1);
    }
}

screenblock::~screenblock ()
{
  if (cmap)
    delete (cmap);
}

// ********* Image Description *******************  
imagedesc::imagedesc (gifblock * block, gifblock * sb, gifblock * pimg,
		      gifblock * gctrl, char verbose, GifFileType * GifFile):
gifblock (block)
{
// Member init
  l = GifFile->Image.Left;
  t = GifFile->Image.Top;
  w = GifFile->Image.Width;
  h = GifFile->Image.Height;
  i = GifFile->Image.Interlace;
  n = GifFile->ImageCount;
  (void *) qpa = (void *) cmap = (void *) pix = NULL;
  scrblk = (screenblock *) sb;
  prev = (imagedesc *) pimg;

  verb = verbose;
  gcontrol = (graphicsblock *) gctrl;

  qpa = new QPointArray (w);
  if (!qpa)
    throw (-1);

// Colormap Init  
  if (GifFile->Image.ColorMap)
    {
      cmap = new colormap (GifFile->Image.ColorMap);	// Anzahl der Farben
      if (!cmap)
	throw (-1);
    }

// Pixel Init
  pix = new pixel[w * h];
  if (!pix)
    throw (-1);

  // Ob das geht ?       
  if (DGifGetLine (GifFile, (GifPixelType *) pix, w * h) == GIF_ERROR)
    {
      //PrintGifError ();
      //exit (-1);
      throw (1);
    }
}

imagedesc::~imagedesc ()
{
  if (cmap)
    delete cmap;

  if (qpa)
    delete[]qpa;

  if (pix)
    delete[]pix;
}

unsigned int
imagedesc::getnr ()
{
  return n;
}

/******************************************************************************
* Print the given PixelBlock of length Len.				      *
* Save local information so printing can be performed continuously,           *
* or reset to start state if Reset. If PixelBlock is NULL, output is flushed  *
******************************************************************************/
void
imagedesc::printpixelblock ()
{
  int CrntPlace = 0;
  long ExtCount = 0;
  char HexForm[49], AsciiForm[17];

  for (int x = 0; x < w * h; x++)
    {
      sprintf (&HexForm[CrntPlace * 3], " %02x", (unsigned char) (pix[x].p));
      sprintf (&AsciiForm[CrntPlace], "%c",
	       MAKE_PRINTABLE ((unsigned char) pix[x].p));
      if (++CrntPlace == 16)
	{
	  CrntPlace = HexForm[CrntPlace * 3] = AsciiForm[CrntPlace] = 0;
	  cout.form ("\n%05lx: %-49s  %-17s", ExtCount, HexForm, AsciiForm);
	  ExtCount += 16;
	}
    }
}

// *********** Kommentar *****************
commentblock::commentblock (gifblock * block, GifByteType * ext):
gifblock (block)
{
  len = ext[0];
  str = NULL;

  str = new char[len + 1];
  if (!str)
    throw (-1);

  memcpy (str, (unsigned char *) &ext[1], sizeof (char) * len);
  str[len + 1] = 0;
}

commentblock::~commentblock ()
{
  if (str)
    delete[]str;
}

void
commentblock::draw (void *widget, void *win)
{
  // die Anzeige des Kommentars ist Sache der Applikation
}
unsigned int
commentblock::getnr ()
{
  return 0;
}


// ************** Graphics Block *************************
graphicsblock::graphicsblock (gifblock * block, unsigned int nt,
GifByteType * ext):
gifblock (block)
{
  grbl = (gb *) ext;

  bs = grbl->bs;

  // User kann per Option eine andere Newtime angeben.  
  dt = (nt) ? nt : (grbl->dt2 << 8 | grbl->dt1);

  tci = grbl->tc;

  tc = grbl->pf & 0x1;
  ui = (grbl->pf & 0x2) >> 1;
  dm = (grbl->pf & 0x1C) >> 2;
}

graphicsblock::~graphicsblock ()
{
}

void
graphicsblock::draw (void *widget, void *win)
{
  // *** zeichnet nix, sind nur Daten fr nachfolgendes Bild ****
}

unsigned int
graphicsblock::getnr ()
{
  return 0;
}


// ************** Text Block *************************
ptextblock::ptextblock (gifblock * block, gifblock * scrblock,
GifByteType * ext):
gifblock (block)
{
  txtblk = (plaintb *) ext;

  bs = txtblk->bs;		// Block Size    
  tglp = txtblk->tglp2 << 8 | txtblk->tglp1;	// Text Grid Left Position 
  tgtp = txtblk->tgtp2 << 8 | txtblk->tgtp1;	// Text Grid Top Position 
  tgw = txtblk->tgw2 << 8 | txtblk->tgw1;	// Text Grid Width
  tgh = txtblk->tgh2 << 8 | txtblk->tgh1;	// Text Grid Height
  ccw = txtblk->ccw;		// Character Cell Width
  cch = txtblk->cch;		// Character Cell Height
  fg = txtblk->fg;		// Text Foreground Color Index
  bg = txtblk->bg;

  // globale Farbtabelle
  if (scrblock)
    {
      fgc = ((screenblock *) scrblock)->cmap->getcolor (fg);
      bgc = ((screenblock *) scrblock)->cmap->getcolor (bg);
    }
  else
    {
      fgc.r = fgc.g = fgc.b = bgc.r = bgc.g = bgc.b = 0;
    }
}


ptextblock::~ptextblock ()
{				// *** zeichnet nix, sind nur Daten fr nachfolgenden TextBlock ****
}

// ********** Text Block ************
void
ptextblock::draw (void *widget, void *win)
{
}

unsigned int
ptextblock::getnr ()
{
  return 0;
}

// ************** Text Str Block *************************
pstrblock::pstrblock (gifblock * block, gifblock * beschblk,
GifByteType * ext):
gifblock (block)
{
  txtblk = (plaintb *) ext;

  bblk = (ptextblock *) beschblk;	// Vorgnger, Beschreibungsblock

  // Text Sub Block
  bs = txtblk->bs;

  ptstr = new char[ext[1] + 1];	// 2 ? weis nicht
  if (!ptstr)
    throw (-1);

  memcpy (ptstr, (char *) &ext[1], bs * sizeof (char));
  ptstr[bs] = 0;
}


pstrblock::~pstrblock ()
{
  if (ptstr)
    delete[]ptstr;
}

unsigned int
pstrblock::getnr ()
{
  return 0;
}


// ******* Appblock ***************************************
appblock::appblock (gifblock * block, GifByteType * ext):
gifblock (block)
{
  appbl *tb = (appbl *) ext;	// Standart 
  loopblock *tbl = (loopblock *) ext;	// Netscape Loop

  bs = tb->bs;
  memcpy (str, tb->indent, 8);
  str[bs] = 0;

  a1 = tb->a1;
  a2 = tb->a2;
  a3 = tb->a3;

  len = ext[12];

  if (tbl->bs == 3 && tbl->kennung == 1)
    {
      anzahl = tbl->anz;
      len = tbl->kennung;
      bs = tbl->bs;
    }
  else
    anzahl = 0;

}

appblock::~appblock ()
{

}

void
appblock::draw (void *widget, void *win)	// Zeichenfunktion
{
}

unsigned int
appblock::getnr ()
{
  return 0;
}
